#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSelection.h>

/*
   Should not be here...
*/
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoCoordinate3.h>

/*
 Xt Components
*/
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/SbViewportRegion.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoLightModel.h>

#include <Inventor/manips/SoHandleBoxManip.h>
#include <Inventor/manips/SoTrackballManip.h>

#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>

#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/SoOutput.h>

#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#include <assert.h>
#include "ModelInvPts.h"
#include "NewInterPlann.h"
#include "CoordSystem.h"
#include "Axis.h"

SoXtExaminerViewer *MainNpExaminerViewer;  // The examiner viewer
SoSeparator *SceneRoot;  // Root of our scene
Model *_theModel;
M_CoordSystem *CoordS=NULL;




//Functions from other modules
//Model *
//returnModel(FILE *fpts, int numPtsToRead, int &EndOfFile);
void
reset_seq_of_points();
int
read_seq_of_points(Model *&theModel, FILE *fpts, int numPtsToRead);
int
read_seq_of_points2(Model *&theModel, FILE *fpts, int numPtsToRead); //points w/ 4 coordinates
int
read_seq_of_pointsCYRA(Model *&theModel, FILE *fpts, int numPtsToRead);
int
read_seq_of_pointsCYRA_wTRANSF(Model *&theModel, FILE *fpts, int numPtsToRead);

//Functions of this module
SoSeparator *
get_model_root(FILE *f_pts, int numPtsToRead, Model *the_model);
void
put_lights(SoSeparator *sceneRoot);
void
XtCallbackFunction( void *, const DataToApp *interface); //User interaction
void
createCoordSystem (SoSeparator *root);
void
printIvFile(char *file_name);

//Globals
FILE *_f_pts;

int _numPtsToRead;
int _is_cyra_file=0;
int _connect_with_lines=0;
double _r_rgb=0.0;
double _g_rgb=0.0;
double _b_rgb=0.0;

main(int argc, char **argv){
    int curr_file=7;
    if (argc<8) {
	printf("Usage: %s  num_of_pts_per_iteration <cyra?> <connect with lines?> <R> <G> <B> [input_file_name]*\n",argv[0]);
	printf("<cyra?>: 0 (ptx), 1(cloud), 2(cloud with 4th coordinate\n");
	return(0);
    }

//     if ( (_f_pts=fopen(argv[1], "r"))==NULL ){
// 	printf("Can't open file %s for reading\n", argv[1]);
// 	exit(0);
//     }
    _numPtsToRead=atoi( argv[1] );
    _is_cyra_file=atoi( argv[2] ); //0 (pts) 1 (cyra) 2 (cyra w. transf)
    _connect_with_lines=atoi( argv[3] );
    _r_rgb=atof( argv[4] );
    _g_rgb=atof( argv[5] );
    _b_rgb=atof( argv[6] );


    Widget myWindow = SoXt::init(argv[0]);
    if (myWindow == NULL) exit(1);



    SoXtExaminerViewer *MainExaminerViewer=new SoXtExaminerViewer( myWindow );
    SceneRoot = new SoSeparator;
    SceneRoot->ref();
    put_lights(SceneRoot);
    M_InterPlann *myXtComponent = new M_InterPlann;
    myXtComponent->addScaleChangedCallback(XtCallbackFunction ,MainNpExaminerViewer);


    while(curr_file<argc){
	Model *the_model=new Model;
	_theModel=the_model;
	if ( (_f_pts=fopen(argv[curr_file], "r"))==NULL ){
	    printf("Can't open file %s for reading\n", argv[curr_file]);
	} else{
	    reset_seq_of_points(); //num of read points =0
	    if (_r_rgb==0 && _g_rgb==0 && _b_rgb==0) the_model->RandomColor();
	    else the_model->setColor(_r_rgb, _g_rgb, _b_rgb);

	    SceneRoot->addChild(get_model_root(_f_pts, _numPtsToRead, the_model));
	}
	curr_file++;
	fclose(_f_pts);
    }



    //Xt=connection




    //
    MainExaminerViewer->setSceneGraph( SceneRoot );
    MainExaminerViewer->show();
/*
  -- Display main window
*/
    SoXt::show(myWindow);

    //--
    myXtComponent->setSceneGraph( SceneRoot );
    myXtComponent->show();
    //<<

    SoXt::mainLoop();

}

//==Interface with the X module
void
read_next_set()
{
    if (_is_cyra_file==0)
	read_seq_of_pointsCYRA(_theModel, _f_pts, _numPtsToRead);
    else if (_is_cyra_file==1)
	read_seq_of_points(_theModel, _f_pts, _numPtsToRead);
    else if (_is_cyra_file==2)
	read_seq_of_points2(_theModel, _f_pts, _numPtsToRead);
}


void
XtCallbackFunction( void *, const DataToApp *interface){

    if (interface->is_ReadNext()==TRUE){
	read_next_set();
    } else if (interface->is_ShowCoordSystem()==TRUE){
	createCoordSystem (SceneRoot);
    } else if (interface->is_printIvFile()==TRUE){
	printIvFile("THE_SCENE.iv");
    }



}


SoSeparator *
get_model_root(FILE *f_pts, int numPtsToRead, Model *the_Model){

    if (_is_cyra_file)
	read_seq_of_pointsCYRA(the_Model, _f_pts, _numPtsToRead);
    else
	read_seq_of_points(the_Model, f_pts, numPtsToRead);
    return the_Model->getRoot();

}


void
put_lights(SoSeparator *sceneRoot)
{
//    SoSeparator *LightRoot = new SoSeparator;

    SoDirectionalLight *mydLight=new SoDirectionalLight;
    mydLight->direction.setValue(1, 1.5, 0.5);

    SoDirectionalLight *mypLight=new SoDirectionalLight;
    mypLight->direction.setValue( -0.7, -0.4 , 0.1 );

    SoDirectionalLight *myxLight=new SoDirectionalLight;
    mypLight->direction.setValue( 0.3, -0.5, 2.0 );

    sceneRoot->addChild(mydLight);
    sceneRoot->addChild(mypLight);
    sceneRoot->addChild(myxLight);

    SoLightModel *myModel=new SoLightModel;
    myModel->model = SoLightModel::PHONG;
    sceneRoot->addChild(myModel);

 //   return LightRoot;
}



//Copied from NewModel.C
void
createCoordSystem (SoSeparator *root) {

  printf("create coord system...\n");
 float smax;
  /*  
SbViewportRegion VpRegion = MainNpExaminerViewer->getViewportRegion();
  SoGetBoundingBoxAction *BBaction =
      new  SoGetBoundingBoxAction( VpRegion );
  assert(BBaction!=0);



  BBaction->apply( root );


  float sx, sy, sz;
 
  SbBox3f bbox = BBaction->getBoundingBox();

  bbox.getSize(sx, sy, sz);
  printf("size= %f %f %f\n", sx, sy, sz);

  if (sx > sy) smax=sx;
     else smax=sy;
  if (sz > smax) smax=sz;

  */
  smax=50;
  printf("max: %f\n", smax);
  if (CoordS==NULL){
      CoordS= new M_CoordSystem( smax, 10);
      assert(CoordS!=0);
    }
  printf("Now I will create the axis...\n");
  M_Axis *m_Axis= new M_Axis;
  assert(m_Axis!=0);
  SoSeparator *axisSep=m_Axis->get_Root();
  printf("I got the coord. sys. root...\n");
  root->addChild( axisSep );

  // delete BBaction;
}

void
printIvFile(char *file_name)
{
    FILE *fpw=fopen(file_name, "w+");
    if (fpw!=NULL){
	SoOutput *newOut=new SoOutput;
	newOut->setFilePointer( fpw );
	SoWriteAction newWrite( newOut );
	newWrite.apply(SceneRoot);
	delete newOut;
	fclose (fpw);
    } else fprintf(stderr, "Can't open file %s to save IV file\n",
		   file_name);
}
